#! /bin/sh

### Boot OpenVPN ##############################################################
#                                                                             #
#  Usage : bootovpn {start|stop}                                              #
#                                                                             #
###############################################################################


# =============================================================================
# Definition
# =============================================================================
#
# ----- define common variables and functions -----
. /usr/local/richscripts/etc/common.lib


# =============================================================================
# Parse the arguments
# =============================================================================
#
case x$1 in
  xstart) ;;
  xstop)  ;;
  *)      echo >&2 "Usage: $0 {start|stop}"
esac


# =============================================================================
# SUBROUTINE: Load the TAP module
# -----------------------------------------------------------------------------
#  arg1  : OS type(the return string of `uname`)
#  return: none
# =============================================================================
#
load_module() {
  case "$1" in
    FreeBSD)
      # use the module loding script if given by Ports
      # else load the module by myself
      loading_module_script_by_ports="/usr/local/etc/rc.d/openvpn.sh.sample"
      if [ -f "$loading_module_script_by_ports" ]; then
        /bin/sh $loading_module_script_by_ports start
      else
        /sbin/kldload if_tap
      fi
      ;;
    *)
      # Not unspported yet
      ;;
  esac
}


# =============================================================================
# SUBROUTINE: Unload the TAP module
# -----------------------------------------------------------------------------
#  arg1  : OS type(the return string of `uname`)
#  return: none
# =============================================================================
#
unload_module() {
  case "$1" in
    FreeBSD)
      # use the module loding script if given by Ports
      # else load the module by myself
      loading_module_script_by_ports="/usr/local/etc/rc.d/openvpn.sh.sample"
      if [ -f "$loading_module_script_by_ports" ]; then
        /bin/sh $loading_module_script_by_ports stop
      else
        /sbin/kldunload if_tap
      fi
      ;;
    *)
      # Not unspported yet
      ;;
  esac
}


# =============================================================================
# SUBROUTINE: Get the Process ID of "the" OpenVPN
# -----------------------------------------------------------------------------
#  arg1  : none
#  return: Process ID of "the" OpenVPN
#          or 0 when "the" OpenVPN process does not exist
# =============================================================================
#
get_pid_of_openvpn() {
  local pid
  for pid in \
  `$CMD_PS axww | $CMD_GREP $CMD_OPENVPN | $CMD_GREP $PATH_OPENVPN_CONF`; do
    break
  done
  [ -z "$pid" ] && pid=0
  return $pid
}


# =============================================================================
# Load or Unload the TAP module And Start or STOP OpenVPN
# =============================================================================
#
case "$1" in
  start)
    # ----- Load the TAP Module & Start OpenVPN -------------------------------
    load_module "$OS_TYPE"
    get_pid_of_openvpn
    pid=$?
    if [ "$pid" -ne 0 ]; then
      $CMD_KILL $pid
      retry=30
      while [ "$retry" -gt 0 ]; do
        $CMD_SLEEP 1
        get_pid_of_openvpn
        pid=$?
        [ "$pid" -ne 0 ] && break
        retry=$(( $retry - 1 ))
      done
    fi
    if [ $retry -le 0 ]; then
      unload_module "$OS_TYPE"
      exit 1
    fi
    $CMD_OPENVPN --config $PATH_OPENVPN_CONF --daemon
    [ -n "$i2if" ] && $PATH_LUWAITER "$i2if"
    ;;
  stop)
    # ----- Stop OpenVPN & Unload the TAP Module ------------------------------
    get_pid_of_openvpn
    pid=$?
    if [ "$pid" -ne 0 ]; then
      $CMD_KILL $pid
      retry=30
      while [ "$retry" -gt 0 ]; do
        $CMD_SLEEP 1
        get_pid_of_openvpn
        pid=$?
        [ "$pid" -ne 0 ] && break
        retry=$(( $retry - 1 ))
      done
    fi
    unload_module "$OS_TYPE"
    ;;
esac
