#! /bin/sh

if [ `uname` != "FreeBSD" ]; then
    echo "This script is for only FreeBSD."
    exit 1
fi

version=`uname -r`
version=${version%%.*}
if [ $version -le 4 ]; then
    PATH_SAVER_MODULES_DIR="/modules"
else
    PATH_SAVER_MODULES_DIR="/boot/kernel"
fi
FILE_PATTERN_SAVER_MODULES="_saver.ko"
VIDDEV="/dev/ttyv0"

for arg in $*
do
    if [ -z "$arg_switch" ]; then
        if [ "$arg" = "-l" ]; then
            print_list="YES"
        else
            arg_switch="$arg"
        fi
    elif [ "$arg_switch" = "-b" ]; then
        arg_switch=""
        blanktime="$arg"
    elif [ "$arg_switch" = "-s" ]; then
        arg_switch=""
        chosen_saver="$arg"
    fi
done

case $chosen_saver in
[Nn][Oo])
    chosen_saver="NO"
    ;;
*)
    ;;
esac

# Set the term until running a screen saver
if [ -n "$blanktime" ]; then
    i="blank_time_is_set"
fi
blanktime=`echo $blanktime | awk '/^\+*[0-9]+$/ {if ($'0'>0) print $'0'}'`
if [ -n "$blanktime" ]; then
    vidcontrol < ${VIDDEV} -t ${blanktime}
    if [ $? = "0" ]; then
        echo \
        "Waiting time until running a saver was changed. -> ${blanktime}sec"
    fi
elif [ "$i" = "blank_time_is_set" ]; then
    echo "Invalid number with waiting_seconds!"
fi

# Search the screen savers
if [ "$print_list" = "YES" -o -n "$chosen_saver" ]; then
    for PATH_SAVER_SOURCE in \
    `find $PATH_SAVER_MODULES_DIR -name "*$FILE_PATTERN_SAVER_MODULES"`
    do
        SAVER_LIST="${SAVER_LIST}\
        `basename $PATH_SAVER_SOURCE $FILE_PATTERN_SAVER_MODULES` "
    done
fi

# List the screen savers
if [ "$print_list" = "YES" ]; then
    echo "*** The screen savers you can choose are as follows."
    for saver in $SAVER_LIST
    do
        echo " ${saver}"
    done
fi

# Register the new screen saver
if [ -n "$chosen_saver" ]; then
    # Make sure the chosen saver is in the searched list
    for saver in $SAVER_LIST
    do
        if [ "$chosen_saver" = "$saver" ]; then
            chosen_saver_existance="YES"
        fi
    done
    if [ -n "$chosen_saver_existance" -o "$chosen_saver" = "NO" ]; then
        # Unload the current screen saver
        i=""
        for i in \
        `kldstat | awk '/_saver\.ko/ {split($'5',svr,"_"); print svr[1]}'`
        do
            kldunload ${i}_saver
            if [ $? = "0" ]; then
                echo "Unload the current saver. ($i)"
            fi
        done
        if [ -z "$i" -a "$chosen_saver" = "NO" ]; then
            echo "No screen saver is registered."
        fi
        if [ -n "$chosen_saver_existance" ]; then
            # Load the new screen saver
            i=`kldstat -v | grep _saver`
            if [ -z $i ]; then
                kldload ${chosen_saver}_saver
                if [ $? = "0" ]; then
                    echo "New screen saver is set. (${chosen_saver})"
                fi
            else
                echo "Failed to register the new screen saver..."
            fi
        fi
    else
        echo "The screen saver you chosen does not exist!"
        chosen_saver=""
    fi
fi

# Help message
if [ -z "$print_list" -a -z "$chosen_saver" -a -z "$blanktime" ]; then
    echo \
    "Usage: saver.sh [-l] [-b waiting_seconds] [-s {screen_saver_name | NO}]"
fi
